@@ -754,7 +754,7 @@ rspamd_mime_header_decode(rspamd_mempool_t *pool, const char *in,
 						state = parse_normal;
 					}
 				} /* qmarks >= 3 */
-			}     /* p == '=' */
+			} /* p == '=' */
 			else {
 				state = got_encoded_start;
 			}
@@ -807,7 +807,9 @@ rspamd_mime_header_decode(rspamd_mempool_t *pool, const char *in,
 
 	g_byte_array_free(token, TRUE);
 	g_byte_array_free(decoded, TRUE);
+	/* Replace control chars and ensure valid UTF-8 */
 	rspamd_mime_header_sanity_check(out);
+	rspamd_mime_charset_utf_enforce(out->str, out->len);
 	rspamd_mempool_notify_alloc(pool, out->len);
 	ret = g_string_free(out, FALSE);
 	rspamd_mempool_add_destructor(pool, g_free, ret);
@@ -816,88 +818,220 @@ rspamd_mime_header_decode(rspamd_mempool_t *pool, const char *in,
 }
 
 char *
-rspamd_mime_header_encode(const char *in, gsize len)
+rspamd_mime_header_encode(const char *in, gsize len, bool is_structured)
 {
-	const char *p = in, *end = in + len;
-	char *out, encode_buf[80 * sizeof(uint32_t)];
-	GString *res;
-	gboolean need_encoding = FALSE;
+	static const size_t max_token_size = 76 - 12; /* 12 is the length of "=?UTF-8?Q??="; */
+	GString *outbuf = g_string_sized_new(len);
+	char *encode_buf = g_alloca(max_token_size + 3);
+	const char *p = in;
+	const char *end = in + len;
+	/* Accumulate pending whitespace between segments to embed into next encoded-word */
+	size_t pending_spaces = 0, pending_tabs = 0;
 
-	/* Check if we need to encode */
 	while (p < end) {
-		if ((((unsigned char) *p) & 0x80) != 0) {
-			need_encoding = TRUE;
-			break;
+		/* Collect linear white space to possibly include into next encoded-word */
+		if (*p == ' ') {
+			pending_spaces++;
+			p++;
+			continue;
 		}
-		p++;
-	}
+		else if (*p == '\t') {
+			pending_tabs++;
+			p++;
+			continue;
+		}
+		else if (*p == '\r' || *p == '\n') {
+			/* Flush pending WS before hard newlines */
+			while (pending_spaces--) {
+				g_string_append_c(outbuf, ' ');
+			}
+			pending_spaces = 0;
+			while (pending_tabs--) {
+				g_string_append_c(outbuf, '\t');
+			}
+			pending_tabs = 0;
+			g_string_append_c(outbuf, *p);
+			p++;
+			continue;
+		}
+		else if (*p == '(' || *p == ')') {
+			/* Flush pending WS around CFWS delimiters */
+			while (pending_spaces--) {
+				g_string_append_c(outbuf, ' ');
+			}
+			pending_spaces = 0;
+			while (pending_tabs--) {
+				g_string_append_c(outbuf, '\t');
+			}
+			pending_tabs = 0;
+			g_string_append_c(outbuf, *p);
+			p++;
+			continue;
+		}
+		else {
+			/* Decide whether we start an encoded span right away */
+			unsigned char first_c = (unsigned char) *p;
+			gboolean starts_encoding = (first_c >= 128) || (is_structured && !g_ascii_isalnum(first_c));
 
-	if (!need_encoding) {
-		out = g_malloc(len + 1);
-		rspamd_strlcpy(out, in, len + 1);
-	}
-	else {
-		/* Need encode */
-		gsize ulen, pos;
-		int r;
-		const char *prev;
-		/* Choose step: =?UTF-8?Q?<qp>?= should be less than 76 chars */
-		unsigned int step = (76 - 12) / 3 + 1;
+			const char *piece_end = end;
+			size_t piece_len = piece_end - p;
+			gboolean need_encoding = FALSE;
+			size_t unencoded_prefix = 0;
+			size_t encoded_len_count = 0;
+			size_t enc_span = 0;
+			gboolean include_pending_ws = starts_encoding && (pending_spaces > 0 || pending_tabs > 0);
+			size_t pending_ws_budget = include_pending_ws ? (pending_spaces + pending_tabs * 3) : 0;
 
-		ulen = g_utf8_strlen(in, len);
-		res = g_string_sized_new(len * 2 + 1);
-		pos = 0;
-		prev = in;
-		/* Adjust chunk size for unicode average length */
-		step *= 1.0 * ulen / (double) len;
+			/* Determine how much of this piece needs encoding and fits the budget */
+			for (size_t i = 0; i < piece_len; i++) {
+				unsigned char c = p[i];
 
-		while (pos < ulen) {
-			p = g_utf8_offset_to_pointer(in, pos);
+				if (!need_encoding) {
+					if (c >= 128 || (is_structured && !g_ascii_isalnum(c))) {
+						need_encoding = TRUE;
+						/* Start encoded region with this char */
+						size_t add = (g_ascii_isalnum(c) || c == ' ') ? 1 : 3;
 
-			if (p > prev) {
-				/* Encode and print */
-				r = rspamd_encode_qp2047_buf(prev, p - prev,
-											 encode_buf, sizeof(encode_buf));
+						if (add + pending_ws_budget > max_token_size) {
+							/* Nothing fits, stop here to emit prefix only */
+							piece_len = i;
+							piece_end = p + piece_len;
+							break;
+						}
 
-				if (r != -1) {
-					if (res->len > 0) {
-						rspamd_printf_gstring(res, " =?UTF-8?Q?%*s?=", r,
-											  encode_buf);
+						encoded_len_count = pending_ws_budget + add;
+						enc_span = 1;
 					}
 					else {
-						rspamd_printf_gstring(res, "=?UTF-8?Q?%*s?=", r,
-											  encode_buf);
+						/* Still in unencoded prefix */
+						unencoded_prefix++;
 					}
 				}
-			}
-
-			pos += MIN(step, ulen - pos);
-			prev = p;
-		}
-
-		/* Leftover */
-		if (prev < end) {
-			r = rspamd_encode_qp2047_buf(prev, end - prev,
-										 encode_buf, sizeof(encode_buf));
-
-			if (r != -1) {
-				if (res->len > 0) {
-					rspamd_printf_gstring(res, " =?UTF-8?Q?%*s?=", r,
-										  encode_buf);
-				}
 				else {
-					rspamd_printf_gstring(res, "=?UTF-8?Q?%*s?=", r,
-										  encode_buf);
+					/* Also stop on parentheses to keep CFWS outside */
+					if (c == '(' || c == ')') {
+						piece_len = i;
+						piece_end = p + piece_len;
+						break;
+					}
+
+					/* Break on whitespace to keep spaces outside encoded-words */
+					if (c == ' ' || c == '\t') {
+						piece_len = i;
+						piece_end = p + piece_len;
+						break;
+					}
+
+					/* For non-structured, include ASCII punctuation only if bridging to non-ASCII ahead */
+					if (!is_structured && c < 128 && !g_ascii_isalnum(c)) {
+						gboolean bridge_to_non_ascii = FALSE;
+						for (size_t j = i + 1; j < piece_len; j++) {
+							unsigned char nc = p[j];
+							if (nc >= 128) {
+								bridge_to_non_ascii = TRUE;
+								break;
+							}
+							if (g_ascii_isspace(nc) || nc == '(' || nc == ')') {
+								break;
+							}
+						}
+
+						if (!bridge_to_non_ascii) {
+							piece_len = i;
+							piece_end = p + piece_len;
+							break;
+						}
+					}
+
+					size_t add = (g_ascii_isalnum(c) || c == ' ') ? 1 : 3;
+
+					if (encoded_len_count + add > max_token_size) {
+						/* Budget exceeded; stop encoded span before this char */
+						piece_len = i;
+						piece_end = p + piece_len;
+						break;
+					}
+
+					encoded_len_count += add;
+					enc_span++;
 				}
 			}
-		}
 
-		out = g_string_free(res, FALSE);
+			if (need_encoding && enc_span > 0) {
+				/* Emit prefix; if we are not starting encoding, flush pending WS literally */
+				if (!include_pending_ws && (pending_spaces > 0 || pending_tabs > 0)) {
+					while (pending_spaces--) {
+						g_string_append_c(outbuf, ' ');
+					}
+					pending_spaces = 0;
+					while (pending_tabs--) {
+						g_string_append_c(outbuf, '\t');
+					}
+					pending_tabs = 0;
+				}
+				g_string_append_len(outbuf, p, unencoded_prefix);
+				p += unencoded_prefix;
+
+				/* Encode encoded span safely within budget */
+				g_string_append(outbuf, "=?UTF-8?Q?");
+
+				/* Prepend pending whitespace inside encoded-word if any */
+				if (include_pending_ws) {
+					for (size_t i = 0; i < pending_spaces; i++) {
+						g_string_append_c(outbuf, '_');
+					}
+					for (size_t i = 0; i < pending_tabs; i++) {
+						g_string_append_len(outbuf, "=09", 3);
+					}
+					pending_spaces = 0;
+					pending_tabs = 0;
+				}
+
+				size_t out_budget = max_token_size - (include_pending_ws ? pending_ws_budget : 0);
+				gssize enc_written = rspamd_encode_qp2047_buf(p, enc_span,
+															  encode_buf, out_budget);
+
+				if (G_UNLIKELY(enc_written < 0)) {
+					/* Extremely conservative fallback: shrink until it fits */
+					while (enc_span > 0 && enc_written < 0) {
+						enc_span--;
+						enc_written = rspamd_encode_qp2047_buf(p, enc_span,
+															   encode_buf, max_token_size);
+					}
+				}
+
+				if (enc_span > 0 && enc_written >= 0) {
+					g_string_append_len(outbuf, encode_buf, (size_t) enc_written);
+					g_string_append(outbuf, "?=");
+					p += enc_span;
+				}
+
+				/* Do not append any suffix here; remaining bytes will be handled next loop */
+			}
+			else {
+				/* No encoding needed or nothing to encode */
+				/* Flush pending whitespace literally before ASCII chunk */
+				if (pending_spaces > 0 || pending_tabs > 0) {
+					while (pending_spaces--) {
+						g_string_append_c(outbuf, ' ');
+					}
+					pending_spaces = 0;
+					while (pending_tabs--) {
+						g_string_append_c(outbuf, '\t');
+					}
+					pending_tabs = 0;
+				}
+				g_string_append_len(outbuf, p, piece_len);
+				p += piece_len;
+			}
+		}
 	}
 
-	return out;
+	/* return the allocated string and free the GString struct */
+	return g_string_free(outbuf, FALSE);
 }
 
+
 char *
 rspamd_mime_message_id_generate(const char *fqdn)
 {
@@ -1438,4 +1572,4 @@ gsize rspamd_strip_smtp_comments_inplace(char *input, gsize len)
 	}
 
 	return (d - start);
-}
\ No newline at end of file
+}
