@@ -18,11 +18,21 @@
 
 #include "lua_common.h"
 
-LUA_PUBLIC_FUNCTION_DEF (config, add_radix_map);
-LUA_PUBLIC_FUNCTION_DEF (config, radix_from_config);
-LUA_PUBLIC_FUNCTION_DEF (config, add_map);
-LUA_PUBLIC_FUNCTION_DEF (config, add_hash_map);
-LUA_PUBLIC_FUNCTION_DEF (config, add_kv_map);
-LUA_PUBLIC_FUNCTION_DEF (config, add_map);
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+LUA_PUBLIC_FUNCTION_DEF(config, add_radix_map);
+LUA_PUBLIC_FUNCTION_DEF(config, radix_from_config);
+LUA_PUBLIC_FUNCTION_DEF(config, radix_from_ucl);
+LUA_PUBLIC_FUNCTION_DEF(config, add_map);
+LUA_PUBLIC_FUNCTION_DEF(config, add_hash_map);
+LUA_PUBLIC_FUNCTION_DEF(config, add_kv_map);
+LUA_PUBLIC_FUNCTION_DEF(config, add_map);
+LUA_PUBLIC_FUNCTION_DEF(config, get_maps);
+
+#ifdef __cplusplus
+}
+#endif
 
 #endif /* SRC_LUA_LUA_MAP_H_ */
