@@ -1,11 +1,11 @@
-/*-
- * Copyright 2016 Vsevolod Stakhov
+/*
+ * Copyright 2024 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -23,56 +23,50 @@
 #include "libserver/cfg_file.h"
 #include "libserver/rspamd_control.h"
 
-static const guint64 rspamd_worker_magic = 0xb48abc69d601dc1dULL;
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+static const uint64_t rspamd_worker_magic = 0xb48abc69d601dc1dULL;
 
 struct rspamd_lang_detector;
 
 struct rspamd_worker_ctx {
-	guint64 magic;
+	uint64_t magic;
 	/* Events base */
-	struct event_base *ev_base;
+	struct ev_loop *event_loop;
 	/* DNS resolver */
 	struct rspamd_dns_resolver *resolver;
 	/* Config */
 	struct rspamd_config *cfg;
 
-	guint32 timeout;
-	struct timeval io_tv;
+	ev_tstamp timeout;
 	/* Detect whether this worker is mime worker    */
 	gboolean is_mime;
-	/* HTTP worker									*/
-	gboolean is_http;
-	/* JSON output                                  */
-	gboolean is_json;
-	/* Allow learning through worker				*/
-	gboolean allow_learn;
+	/* Allow encrypted requests only using network */
+	gboolean encrypted_only;
 	/* Limit of tasks */
-	guint32 max_tasks;
+	uint32_t max_tasks;
 	/* Maximum time for task processing */
-	gdouble task_timeout;
+	ev_tstamp task_timeout;
 	/* Encryption key */
 	struct rspamd_cryptobox_keypair *key;
 	/* Keys cache */
-	struct rspamd_keypair_cache *keys_cache;
+	struct rspamd_http_context *http_ctx;
 	/* Language detector */
 	struct rspamd_lang_detector *lang_det;
 };
+
 /*
  * Init scanning routines
  */
-void rspamd_worker_init_scanner (struct rspamd_worker *worker,
-		struct event_base *ev_base,
-		struct rspamd_dns_resolver *resolver,
-		struct rspamd_lang_detector **plang_det);
+void rspamd_worker_init_scanner(struct rspamd_worker *worker,
+								struct ev_loop *ev_base,
+								struct rspamd_dns_resolver *resolver,
+								struct rspamd_lang_detector **plang_det);
 
-/*
- * Called on forced timeout
- */
-void rspamd_task_timeout (gint fd, short what, gpointer ud);
-
-/*
- * Called on unexpected IO error (e.g. ECONNRESET)
- */
-void rspamd_worker_guard_handler (gint fd, short what, void *data);
+#ifdef __cplusplus
+}
+#endif
 
 #endif
